class Module
  def cattr_accessor(attribute_name)
    class_eval <<-CODE
      def self.#{attribute_name}
        @@#{attribute_name} ||= nil
      end
      def self.#{attribute_name}=(value)
        @@#{attribute_name} = value
      end
    CODE
  end
end

class Appfront::Command::Base
  cattr_accessor :api

  protected
  def self.echo_off
    with_tty do
      system "stty -echo"
    end
  end

  def self.echo_on
    with_tty do
      system "stty echo"
    end
  end

  def self.with_tty(&block)
    return unless $stdin.isatty
    yield
  end

  def self.ask
    $stdin.gets.to_s.strip
  end

  def self.find_deploy!(opts)
    if opts[:deploy]
      @deploy = opts[:deploy]
    end  

    return true if @deploy

    puts 'No deploy specified.'
    puts 'Specify which deploy to use with --deploy DEPLOY.'
    exit 1
  end

  def self.which(cmd)
    exts = ENV['PATHEXT'] ? ENV['PATHEXT'].split(';') : ['']
    ENV['PATH'].split(File::PATH_SEPARATOR).each do |path|
      exts.each { |ext|
        exe = File.join(path, "#{cmd}#{ext}")
        return exe if File.executable? exe
      }
    end
    return nil
  end

  def self.spinner(desc, &block)
    chars = %w{ | / - \\ }

    t = Thread.new { block.call }
    while t.alive?
      print "#{desc} [#{chars[0]}]\t"
      sleep 0.1
      print "\r"

      chars.push chars.shift
    end

    t.join

    print "#{desc} [ok] \t"
  end
end
